<?php
/**
 * WooCommerce Refunded Orders.
 *
 * Implements methods to get the refunded orders for WooCommerce
 * (complete and partial orders) by $ticket_id.
 *
 * @since 4.7.3
 *
 */
class Tribe__Tickets_Plus__Commerce__WooCommerce__Orders__Refunded {

	/**
	 * @var Tribe__Tickets_Plus__Commerce__WooCommerce__Orders__Refunded[]
	 */
	protected static $instances;

	/**
	 * @var int
	 */
	protected $ticket_id = 0;

	/**
	 * @var int[]
	 */
	protected $count_cache = [];

	/**
	 * Get refunds count. If there's something in cache, then
	 * the cached number, if not run `real_get_count()`.
	 *
	 * @since 4.7.3
	 *
	 * @param      $ticket_id
	 *
	 * @return int
	 */
	public function get_count( $ticket_id ) {

		if ( ! is_numeric( $ticket_id ) ) {
			return;
		}

		$ticket_post = get_post( $ticket_id );
		if ( empty( $ticket_post ) ) {
			return;
		}

		if ( ! isset( $this->count_cache[ $ticket_id ] ) ) {
			$this->count_cache[ $ticket_id ] = $this->real_get_count( $ticket_id );
		}

		return $this->count_cache[ $ticket_id ];
	}

	/**
	 * Get the number of refunds for a ticket
	 * (both complete refunds, plus partial refunds)
	 *.
	 * @since 4.7.3
	 *
	 * @return int
	 */
	protected function real_get_count( $ticket_id ) {
		$args = [
			'status'     => 'refunded', // Fetch only Refunded orders.
			'limit'      => -1, // Fetch all matching orders.
			'meta_query' => [
				[
					'key'     => '_product_id',
					'value'   => $ticket_id,
					'compare' => '=',
				],
			],
		];

		$refunded_orders        = wc_get_orders( $args );
		$refunded_tickets_count = 0;

		foreach ( $refunded_orders as $order ) {
			foreach ( $order->get_items() as $item ) {
				if ( $item->get_product_id() == $ticket_id ) {
					$refunded_tickets_count += $item->get_quantity();
				}
			}
		}

		return $refunded_tickets_count;
	}

	/**
	 * Get the order_item_ids where the ticket is involved (mapped)
	 *
	 * @since 4.7.3
	 * @since 5.9.1 Updated logic to new WooCommerce HPOS requirement.
	 *
	 * @return array
	 */
	protected function get_order_item_ids( $ticket_id ) {
		$args = [
			'limit'      => -1, // Fetch all orders.
			'meta_query' => [
				[
					'key'     => '_product_id',
					'value'   => $ticket_id,
					'compare' => '=',
				],
			],
		];

		$orders         = wc_get_orders( $args );
		$order_item_ids = [];

		foreach ( $orders as $order ) {
			foreach ( $order->get_items() as $item_id => $item ) {
				if ( $item->get_product_id() == $ticket_id ) {
					$order_item_ids[] = $item_id;
				}
			}
		}

		return implode( ',', array_map( 'intval', $order_item_ids ) );
	}

	/**
	 * Get the actual order ids for the ticket, given the mapped values
	 *
	 * @since 4.7.3
	 * @since 5.9.1 Updated logic to new WooCommerce HPOS requirement.
	 *
	 * @return string
	 */
	public function get_order_ids( $order_item_ids_interval ) {
		$order_item_ids = array_map(
			'intval',
			explode(
				',',
				$order_item_ids_interval
			)
		);
		$order_ids      = [];

		foreach ( $order_item_ids as $item_id ) {
			$order_item = new WC_Order_Item_Product( $item_id );
			$order_id   = $order_item->get_order_id();
			if ( $order_id ) {
				$order_ids[] = $order_id;
			}
		}

		return implode( ',', array_unique( $order_ids ) );
	}

	/**
	 * Get the order ids where there was a refund
	 *
	 * @since 4.7.3
	 * @since 5.9.1 Updated logic to new WooCommerce HPOS requirement.
	 *
	 * @return array
	 */
	public function get_refunded_order_post_ids( $order_post_ids_interval ) {
		$order_ids = array_map(
			'intval',
			explode(
				',',
				$order_post_ids_interval
			)
		);

		$refunded_order_ids = [];

		foreach ( $order_ids as $order_id ) {
			$order = wc_get_order( $order_id );

			if ( $order && $order->has_status( 'refunded' ) ) {
				// Check if the order has refunds and add the order ID to the array.
				$refunds = $order->get_refunds();
				if ( ! empty( $refunds ) ) {
					$refunded_order_ids[] = $order_id;
				}
			}
		}

		$refunded_order_ids = array_unique( $refunded_order_ids );

		return $refunded_order_ids;
	}

	/**
	 * Reset the count cache for a specific ticket ID or all tickets.
	 *
	 * @since 5.1.0
	 *
	 * @param null|int $ticket_id The ticket ID to reset or null to reset all.
	 */
	public function reset_count_cache( $ticket_id = null ) {
		if ( null === $ticket_id ) {
			$this->count_cache = [];

			return;
		}

		if ( isset( $this->count_cache[ $ticket_id ] ) ) {
			unset( $this->count_cache[ $ticket_id ] );
		}
	}

}
